/* -*- Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1999 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the MASH Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /opt/simulators/ns/expt/ns-sow/atm/atmnode.h,v 1.0 2000/06/19 17:35:09 sow Exp $
 *
 * Contributed by Sowmya Manjanatha, Dept. of Computer Science, UNH, June 2000 
 */

#include "atmpvc.h"


PVC::PVC(int vp, int vc, AtmTrafprm *rx, AtmTrafprm *tx) {
	vpi = vp;
	vci = vc;
	qos = new AtmQos(rx,tx);

}


int PVC::set_args(int vp, int vc, AtmTrafprm *rx, AtmTrafprm *tx) {
	vpi = vp;
	vci = vc;
	qos = new AtmQos(rx,tx);

	return (TCL_OK);
}


static class pvc_dbClass : public TclClass {
public:
	pvc_dbClass() : TclClass("pvc_db") {}
	TclObject* create(int, const char*const*) {
		return (new pvc_db);
		
	}	
} class_pvc_db;


pvc_db::pvc_db() {
	bind("linknum_",&(resource.num_links));

        for (int i = 0 ; i < MAX_LINKS; i++) {
	    resource.bw[i] = 0.0;
      	    resource.dly[i] = 0.0;
        }
        resource.num_links = 0;
}

int pvc_db::command(int argc, const char*const* argv) {
  Tcl& tcl = Tcl::instance();

               
  if (argc == 5) {

          if (strcmp("add-resources",argv[1]) == 0) {

                 double bw = atof(argv[2]);
                 double dly = atof(argv[3]);
                 int link = atoi(argv[4]);

                 resource.bw[link] = bw;
                 resource.dly[link] = dly;
                 resource.num_links++;

                 return TCL_OK;
          }
  }
  if (argc == 12) {

	  if (strcmp("add-pvc",argv[1]) == 0) {

		  //incoming pvc
		  int vpi = atoi(argv[2]);

		  if(vpi < 0) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[2]);	
			  return TCL_ERROR;
		  }

		  int vci = atoi(argv[3]);

		  if(vci < 0) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[3]);	
			  return TCL_ERROR;
		  }


		  AtmTrafprm *rxtp = (AtmTrafprm*)TclObject::lookup(argv[4]);

		  if(rxtp == NULL) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[4]);	
			  return TCL_ERROR;
		  }

		  
		  AtmTrafprm *txtp = (AtmTrafprm*)TclObject::lookup(argv[5]);		  
		  
		  if(rxtp == NULL) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[5]);	
			  return TCL_ERROR;
		  }


		  PVC p = PVC(vpi,vci,rxtp,txtp);


		  if (strcmp(argv[6],"map") != 0) {
			  tcl.resultf("PVC DATABASE::no outgoing interface to map to");	
			  return TCL_ERROR;
		  };

		  path pth;
		  pth.incoming_pvc = p;

		  //outgoing pvc

		  vpi = atoi(argv[7]);

		  if(vpi < 0) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[7]);	
			  return TCL_ERROR;
		  }

		  vci = atoi(argv[8]);

		  if(vpi < 0) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[8]);	
			  return TCL_ERROR;
		  }

		  rxtp = (AtmTrafprm*)TclObject::lookup(argv[9]);

		  if(rxtp == NULL) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[9]);	
			  return TCL_ERROR;
		  }

		  txtp = (AtmTrafprm*)TclObject::lookup(argv[10]);		  
		  
		  if(txtp == NULL) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[10]);	
			  return TCL_ERROR;
		  }

		  p = PVC(vpi,vci,rxtp,txtp);

		  pth.outgoing_pvc = p;

                  int link_num = atoi(argv[11]);


                   if((link_num < 0) || (link_num > MAX_LINKS)) {
			  tcl.resultf("PVC DATABASE::invalid value %s",argv[11]);	
			  return TCL_ERROR;
		  }

		  // building the table

		  printf("ATMPVC::vpi/vci = %d/%d,scr = %d\n",vpi,vci,txtp->scr);
		  if (!(conditioner->cac(pth,link_num))) {
	                  tcl.resultf("PVC DATABASE::resource not available\n");
			  return TCL_ERROR;
	          }

		  pvc_table.push_back(pth);
		  return (TCL_OK);
	  }
  }

  if (argc == 3) {
	  if(strcmp("attach-conditioner",argv[1]) == 0) {
		  conditioner = (AtmConditioner*)TclObject::lookup(argv[2]);
		  return TCL_OK;
	  }
  }

  return (TclObject::command(argc,argv));
}


PVC *pvc_db::get_outgoing_pvc(int vpi, int vci) {
	
	for(unsigned int i=0; i < pvc_table.size(); i++) {
		int vp = pvc_table[i].incoming_pvc.get_vpi();
		int vc = pvc_table[i].incoming_pvc.get_vci();
		if((vp == vpi) && (vc == vci)) 
			return &(pvc_table[i].outgoing_pvc);
	} 
	return (PVC*)NULL;
}

PVC *pvc_db::get_corresponding_outgoing_pvc(int vpi, int vci) {
	
	for(unsigned int i=0; i < pvc_table.size(); i++) {
		int vp = pvc_table[i].outgoing_pvc.get_vpi();
		int vc = pvc_table[i].outgoing_pvc.get_vci();
		
		if((vp == vpi) && (vc == vci))
			return &(pvc_table[i].outgoing_pvc);
	}
	return (PVC*)NULL;
}

PVC *pvc_db::get_incoming_pvc(int vpi, int vci) {
	
	for(unsigned int i=0; i < pvc_table.size(); i++) {
		int vp = pvc_table[i].incoming_pvc.get_vpi();
		int vc = pvc_table[i].incoming_pvc.get_vci();
		if((vp == vpi) && (vc == vci)) 
			return &(pvc_table[i].incoming_pvc);
	} 
	return (PVC*)NULL;
}



AtmQos *pvc_db::incoming_qos_lookup(int vpi, int vci) {
	
	AtmQos *q;

	for(unsigned int i=0; i < pvc_table.size(); i++) {
		int vp = pvc_table[i].incoming_pvc.get_vpi();
		int vc = pvc_table[i].incoming_pvc.get_vci();

		if((vp == vpi) && (vc == vci)) {
			q = pvc_table[i].incoming_pvc.get_qos();
			return q;
		}

	}

	return (AtmQos *)NULL;
}

AtmQos *pvc_db::outgoing_qos_lookup(int vpi, int vci) {
	

	for(unsigned int i=0; i < pvc_table.size(); i++) {
		int vp = pvc_table[i].outgoing_pvc.get_vpi();
		int vc = pvc_table[i].outgoing_pvc.get_vci();

		if((vp == vpi) && (vc == vci))
			return pvc_table[i].outgoing_pvc.get_qos();

	}

	return (AtmQos *)NULL;
}

path *pvc_db::get_corresponding_table_entry(int vpi, int vci) {

	for(unsigned int i=0; i < pvc_table.size(); i++) {

		int vp = pvc_table[i].outgoing_pvc.get_vpi();
		int vc = pvc_table[i].outgoing_pvc.get_vci();

		if((vpi == vp) && (vci == vc))
			return (path *)(&(pvc_table[i]));
	}

	return (path *)NULL;
}

double pvc_db::get_total_input_rate() {

	double sum = 0.0;

	for (unsigned int i=0; i < pvc_table.size(); i++) 
		sum += pvc_table[i].input_rate;

	return sum;
}
