/* -*- Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1999 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the MASH Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /opt/simulators/ns/expt/ns-sow/atm/atmnode.h,v 1.0 2000/06/19 17:35:09 sow Exp $
 *
 * Contributed by Sowmya Manjanatha, Dept. of Computer Science, UNH, June 2000 
 */

#ifndef __atmpvc_h__
#define __atmpvc_h__

#include "atmqos.h"
#include <vector>
#include "atmconditioner.h"

#define MAX_PVCS 4000
#define MAX_LINKS 10

class PVC : public TclObject {
public:
	PVC() { }
	PVC(int, int,AtmTrafprm *rx, AtmTrafprm *tx);
	int set_args(int, int, AtmTrafprm *rx, AtmTrafprm *tx);
	int get_vpi(){ return vpi ;}
	int get_vci(){ return vci ;}
	AtmTrafprm *get_rxtp() { return qos->get_rxtp(); }
	AtmTrafprm *get_txtp() { return qos->get_txtp(); }
	AtmQos *get_qos() { return qos; }


private:
	int vpi;
	int vci;
	AtmQos *qos;

};

typedef struct path {
	PVC incoming_pvc;
	PVC outgoing_pvc;
	double input_rate;
	double prev_cell_arrival_time;
} path;


typedef struct Resource {
	double bw[MAX_LINKS];
	double dly[MAX_LINKS];
	double num_links;
} Resource;


class pvc_db : public TclObject {
public:
	pvc_db();
	PVC *get_outgoing_pvc(int, int);
	PVC *get_incoming_pvc(int,int);
	PVC *get_corresponding_outgoing_pvc(int,int);
	AtmQos *incoming_qos_lookup(int,int);
	AtmQos *outgoing_qos_lookup(int,int);
	int size() { return pvc_table.size(); }
	path *get_corresponding_table_entry(int,int);
	double get_total_input_rate();
	double avail_rate(int linknum) { return resource.bw[linknum];}
	double line_dly(int linknum) { return resource.dly[linknum];}
	void set_avail_rate(double rt,int linknum) { resource.bw[linknum] -= rt; }
	
	
protected:
	int command(int argc, const char*const* argv);

private:
	vector<path> pvc_table;
	AtmConditioner* conditioner;
	Resource resource;
};
	
#endif








