/* -*- Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1999 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the MASH Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /opt/src/mash/repository/vint/ns-2/atmnode.h,v 1.0 2000/06/19 17:35:09 sow Exp $
 *
 * Contributed by Sowmya Manjanatha, Dept. of Computer Science, UNH, June 2000 
 */


#include "atmqos.h"
#include <ctype.h>

static class AtmTrafprmClass : public TclClass {
public:
	AtmTrafprmClass() : TclClass("AtmTrafprm") {}
	TclObject* create(int, const char*const*) {
		return (new AtmTrafprm);
		
	}	
} class_atmtrafprm;

AtmTrafprm* AtmTrafprm::instance_;



int AtmTrafprm::command(int argc, const char*const* argv) {
	
	Tcl& tcl = Tcl::instance();

	if ((instance_ == 0) || (instance_ != this))
		instance_ = this;

	if (argc == 12) {

		/* pcr, cdvt, peak_2_peak_cdv, max_ctd, clr are the required traffic
                   and qos parameters for CBR service */

		if( strcmp("set-cbr",argv[1]) == 0)
			return set_cbr(argv,tcl);


		if( strcmp("set-nrt-vbr",argv[1]) == 0) {
			return set_nrt_vbr(argv,tcl);
		}
				
	}

	if (argc == 14) {

		if( strcmp("set-gfr",argv[1]) == 0)
			return set_gfr(argv,tcl);

	}

	if (argc == 16) {

		/* pcr, cdvt, peak_2_peak_cdv, max_ctd, clr,scr and mbs are 
		   the required traffic and qos parameters for CBR service */

		if( strcmp("set-rt-vbr",argv[1]) == 0)
			return set_rt_vbr(argv,tcl);
				
	}

	if (argc == 6) {

		if( strcmp("set-ubr",argv[1]) == 0)
			return set_ubr(argv,tcl);		

	}

	if (argc == 10) {

		if( strcmp("set-abr",argv[1]) == 0)
			return set_abr(argv,tcl);		

	}

	return (TclObject::command(argc,argv));
}
	

/* cbr traffic and qos parameters required for cac and upc */

int AtmTrafprm::set_cbr(const char*const* argv,Tcl &tcl) {		

	traffic_class = CBR;

	if (strcmp(argv[2],"pcr") != 0) {
		tcl.resultf("pcr value required for CBR service\n");
		return TCL_ERROR;
	}
	
	// store pcr value
	
	if ((pcr = atoi(argv[3])) < 0) {
		tcl.resultf("invalid pcr value %s\n", argv[3]);
		return TCL_ERROR;
	}
	
	if (strcmp(argv[4], "cdvt") != 0) {
		tcl.resultf("cdvt value required for CBR service\n");
		return TCL_ERROR;
	}
	
	// store cdvt
	
	if ((cdvt = atof(argv[5])) < 0) {
		tcl.resultf("invalid cdvt value %s\n",argv[5]);
		return TCL_ERROR;
	}
	     
	if (strcmp("p2p-cdv",argv[6]) != 0) {
		tcl.resultf("p2p-cdv value required for CBR service\n");
		return TCL_ERROR;	
	}
	
	if ((peak_to_peak_cdv = atof(argv[7])) < 0) {
		tcl.resultf("invalid p2p-cdv value %s\n",argv[7]);
		return TCL_ERROR;
	}

	if (strcmp("max-ctd", argv[8]) != 0) {
		tcl.resultf("max-ctd value required for CBR service\n");
		return TCL_ERROR;	
	}

	if((max_ctd = atof(argv[9])) < 0) {
		tcl.resultf("invalid max_ctd value %s\n",argv[9]);
		return TCL_ERROR;
	}

	if (strcmp("clr", argv[10]) != 0) {
		tcl.resultf("clr value required for CBR service\n");
		return TCL_ERROR;	
	}

	if((clr = atof(argv[11])) < 0) {
		tcl.resultf("invalid clr value %s\n",argv[11]);
		return TCL_ERROR;
	}


	return TCL_OK;
}


/* cbr traffic and qos parameters required for cac and upc */

int AtmTrafprm::set_rt_vbr(const char*const* argv,Tcl &tcl) {		

	traffic_class = rt_VBR;

	if (strcmp(argv[2],"pcr") != 0) {
		tcl.resultf("pcr value required for rt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store pcr value
	
	if ((pcr = atoi(argv[3])) < 0) {
		tcl.resultf("invalid pcr value %s\n", argv[3]);
		return TCL_ERROR;
	}
	
	if (strcmp(argv[4], "cdvt") != 0) {
		tcl.resultf("cdvt value required for rt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store cdvt
	
	if ((cdvt = atof(argv[5])) < 0) {
		tcl.resultf("invalid cdvt value %s\n",argv[5]);
		return TCL_ERROR;
	}
        else
           printf("AtmQos::rt-vbr,cdvt = %f\n",cdvt);
	     
	if (strcmp("scr",argv[6]) != 0) {
		tcl.resultf("scr value required for rt-VBR service\n");
		return TCL_ERROR;	
	}

	// collect scr

	if ((scr = atoi(argv[7])) < 0) {
		tcl.resultf("invalid scr value %s\n",argv[5]);
		return TCL_ERROR;
	}
	     
	if (strcmp("mbs",argv[8]) != 0) {
		tcl.resultf("mbs value required for rt-VBR service\n");
		return TCL_ERROR;	
	}

	// collect mbs

	if ((mbs = atoi(argv[9])) < 0) {
		tcl.resultf("invalid mbs value %s\n",argv[9]);
		return TCL_ERROR;
	}
        else
           printf("AtmQos::rt-vbr, mbs = %d\n",mbs);
	     
	if (strcmp("p2p-cdv",argv[10]) != 0) {
		tcl.resultf("p2p-cdv value required for rt-VBR service\n");
		return TCL_ERROR;	
	}
	
	if ((peak_to_peak_cdv = atof(argv[11])) < 0) {
		tcl.resultf("invalid p2p-cdv value %s\n",argv[11]);
		return TCL_ERROR;
	}

	if (strcmp("max-ctd", argv[12]) != 0) {
		tcl.resultf("max-ctd value required for rt-VBR service\n");
		return TCL_ERROR;	
	}

	if((max_ctd = atof(argv[13])) < 0) {
		tcl.resultf("invalid max_ctd value %s\n",argv[13]);
		return TCL_ERROR;
	}

	if (strcmp("clr", argv[14]) != 0) {
		tcl.resultf("clr value required for rt-VBR service\n");
		return TCL_ERROR;	
	}

	if((clr = atof(argv[15])) < 0) {
		tcl.resultf("invalid clr value %s\n",argv[15]);
		return TCL_ERROR;
	}


	return TCL_OK;
}


/* nrt_vbr traffic and qos parameters required for cac and upc */

int AtmTrafprm::set_nrt_vbr(const char*const* argv,Tcl &tcl) {		

	traffic_class = nrt_VBR;

	if (strcmp(argv[2],"pcr") != 0) {
		tcl.resultf("pcr value required for nrt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store pcr value
	
	if ((pcr = atoi(argv[3])) < 0) {
		tcl.resultf("invalid pcr value %s\n", argv[3]);
		return TCL_ERROR;
	}
	
	if (strcmp(argv[4], "cdvt") != 0) {
		tcl.resultf("cdvt value required for nrt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store cdvt
	
	if ((cdvt = atof(argv[5])) < 0) {
		tcl.resultf("invalid cdvt value %s\n",argv[5]);
		return TCL_ERROR;
	}
        else
           printf("AtmQos::nrt-vbr,cdvt = %f\n",cdvt);
	     
	if (strcmp("scr",argv[6]) != 0) {
		tcl.resultf("scr value required for nrt-VBR service\n");
		return TCL_ERROR;	
	}
	
	if ((scr = atoi(argv[7])) < 0) {
		tcl.resultf("invalid scr value %s\n",argv[7]);
		return TCL_ERROR;
	}

	else {
	     if (scr >= pcr) {
		tcl.resultf("invalid scr value %s\n",argv[7]);
		return TCL_ERROR;
             }
	}


	if (strcmp("mbs", argv[8]) != 0) {
		tcl.resultf("mbs value required for nrt-VBR service\n");
		return TCL_ERROR;	
	}

	if((mbs = atoi(argv[9])) < 0) {
		tcl.resultf("invalid mbs value %s\n",argv[9]);
		return TCL_ERROR;
        }
	else
	   printf("AtmQos::nrt-vbr,mbs = %d\n",mbs);
	

	if (strcmp("clr", argv[10]) != 0) {
		tcl.resultf("clr value required for nrt-VBR service\n");
		return TCL_ERROR;	
	}

	if((clr = atof(argv[11])) < 0) {
		tcl.resultf("invalid clr value %s\n",argv[11]);
		return TCL_ERROR;
	}


	return TCL_OK;
}


/* ubr traffic and qos parameters required for cac and upc */

int AtmTrafprm::set_ubr(const char*const* argv,Tcl &tcl) {		

	traffic_class = UBR;

	if (strcmp(argv[2],"pcr") != 0) {
		tcl.resultf("pcr value required for nrt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store pcr value
	
	if ((pcr = atoi(argv[3])) < 0) {
		tcl.resultf("invalid pcr value %s\n", argv[3]);
		return TCL_ERROR;
	}
	
	if (strcmp(argv[4], "cdvt") != 0) {
		tcl.resultf("cdvt value required for nrt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store cdvt
	
	if ((cdvt = atof(argv[5])) < 0) {
		tcl.resultf("invalid cdvt value %s\n",argv[5]);
		return TCL_ERROR;
	}
	     
	return TCL_OK;
}



/* gfr traffic and qos parameters required for cac and upc */

int AtmTrafprm::set_gfr(const char*const* argv,Tcl &tcl) {		

	traffic_class = GFR;

	if (strcmp(argv[2],"pcr") != 0) {
		tcl.resultf("pcr value required for rt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store pcr value
	
	if ((pcr = atoi(argv[3])) < 0) {
		tcl.resultf("invalid pcr value %s\n", argv[3]);
		return TCL_ERROR;
	}
	
	if (strcmp(argv[4], "cdvt") != 0) {
		tcl.resultf("cdvt value required for rt-VBR service\n");
		return TCL_ERROR;
	}
	
	// store cdvt
	
	if ((cdvt = atof(argv[5])) < 0) {
		tcl.resultf("invalid cdvt value %s\n",argv[5]);
		return TCL_ERROR;
	}
	     
	if (strcmp("mcr",argv[6]) != 0) {
		tcl.resultf("mcr value required for GFR service\n");
		return TCL_ERROR;	
	}

	// collect mcr

	if ((mcr = atoi(argv[7])) < 0) {
		tcl.resultf("invalid mcr value %s\n",argv[7]);
		return TCL_ERROR;
	}
	     
	if (strcmp("mbs",argv[8]) != 0) {
		tcl.resultf("mbs value required for GFR service\n");
		return TCL_ERROR;	
	}

	// collect mbs

	if ((mbs = atoi(argv[9])) < 0) {
		tcl.resultf("invalid mbs value %s\n",argv[9]);
		return TCL_ERROR;
	}
	     
	if (strcmp("mfs",argv[10]) != 0) {
		tcl.resultf("mfs value required for GFR service\n");
		return TCL_ERROR;	
	}
	
	if ((mfs = atoi(argv[11])) < 0) {
		tcl.resultf("invalid mfs value %s\n",argv[11]);
		return TCL_ERROR;
	}

	if (strcmp("clr", argv[12]) != 0) {
		tcl.resultf("clr value required for GFR service\n");
		return TCL_ERROR;	
	}

	if((clr = atof(argv[13])) < 0) {
		tcl.resultf("invalid clr value %s\n",argv[13]);
		return TCL_ERROR;
	}

	return TCL_OK;
}


/* abr traffic and qos parameters required for cac and upc */

int AtmTrafprm::set_abr(const char*const* argv,Tcl &tcl) {		

	traffic_class = ABR;

	if (strcmp(argv[2],"pcr") != 0) {
		tcl.resultf("pcr value required for ABR service\n");
		return TCL_ERROR;
	}
	
	// store pcr value
	
	if ((pcr = atoi(argv[3])) < 0) {
		tcl.resultf("invalid pcr value %s\n", argv[3]);
		return TCL_ERROR;
	}

	acr = pcr;
	
	if (strcmp(argv[4], "cdvt") != 0) {
		tcl.resultf("cdvt value required for ABR service\n");
		return TCL_ERROR;
	}
	
	// store cdvt
	
	if ((cdvt = atof(argv[5])) < 0) {
		tcl.resultf("invalid cdvt value %s\n",argv[5]);
		return TCL_ERROR;
	}
	     
	if (strcmp("mcr",argv[6]) != 0) {
		tcl.resultf("mcr value required for ABR service\n");
		return TCL_ERROR;	
	}

	// collect mcr

	if ((mcr = atoi(argv[7])) < 0) {
		tcl.resultf("invalid mcr value %s\n",argv[7]);
		return TCL_ERROR;
	}
	     

	if (strcmp("clr", argv[8]) != 0) {
		tcl.resultf("clr value required for ABR service\n");
		return TCL_ERROR;	
	}

	if((clr = atof(argv[9])) < 0) {
		tcl.resultf("invalid clr value %s\n",argv[9]);
		return TCL_ERROR;
	}

	return TCL_OK;
}



void AtmTrafprm::initialize() {

	traffic_class = NIL;
	max_pcr = NIL;
	min_pcr = NIL;
	pcr = 0;
	scr = 0;
	mcr = 0;
	mbs = 0;
	mfs = 0;
	cdvt = NIL;
	clr = NIL;
	peak_to_peak_cdv = NIL;
	max_sdu = NIL;
	max_ctd = NIL;
	icr = NIL;
	tbe = NIL;
	frtt = NIL;
	rif = 1/16;
	rdf = 1/16;
	nrm_pres = 32;
	trm_pres = NIL;
	adtf_pres = NIL;
	cdf_pres = NIL;
	nrm = 32;
	trm = NIL;
	cdf = NIL;
	spare = NIL;
	tat = NIL;
}



















AtmQos::AtmQos(AtmTrafprm *rx, AtmTrafprm *tx) {

	rxtp = rx;
	txtp = tx;
}









