/* -*- Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */

/*
 * Copyright (c) 1999 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the MASH Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /opt/src/mash/repository/vint/ns-2/atmnode.h,v 1.0 2000/06/19 17:35:09 sow Exp $
 *
 * Contributed by Sowmya Manjanatha, Dept. of Computer Science, UNH, June 2000 
 */

#ifndef __atmqos_h__
#define __atmqos_h__

#include <tclcl.h>
#include <stdlib.h>

#define CBR 0
#define rt_VBR 1
#define nrt_VBR 2
#define ABR 3
#define UBR 4
#define GFR 5
#define NIL -1

class AtmTrafprm : public TclObject {
public:
	AtmTrafprm() {initialize();}

	/* the parameters below are allowed to be public so that they can 
           be set at configuration time */

        /* parameters generic to most service categories */

	int             traffic_class;  /* traffic class (UBR, ...) */
	int             max_pcr;        /* maximum PCR in cells per second */
	int             pcr;            /* desired PCR in cells per second */
	int             scr;            /* desired SCR in cells per second */
	int             mbs;            /* maximum burst size in cells */
	int             mfs;            /* maximum frame size in cells */
	double          cdvt;           /* cell delay variation tolerance */
	int             min_pcr;        /* minimum PCR in cells per second */
	double          clr;            /* cell loss ratio */
	double          peak_to_peak_cdv;        /* maximum CDV in microseconds */
	int             max_sdu;        /* maximum SDU in bytes */
	double          max_ctd;        /* maximum cell transfer delay */

	/* extra params for ABR */
	
	int    mcr;            /* minimum cell rate in cells/sec */
	int    icr;            /* Initial Cell Rate (24-bit) */
	int    acr;            // Allowed Cell Rate
	int    tbe;            /* Transient Buffer Exposure (24-bit) */
	int    frtt;      /* Fixed Round Trip Time (24-bit) */
	double    rif;       /* Rate Increment Factor */
	double    rdf;       /* Rate Decrease Factor */
	int    nrm_pres;      /* nrm present bit */
	int    trm_pres;      /* rm present bit */
	int    adtf_pres;      /* adtf present bit */
	int    cdf_pres;      /* cdf present bit*/
	int    nrm;      /* Max # of Cells for each forward RM cell (3-bit) */
	int    trm;      /* Time between forward RM cells (3-bit) */
	int    adtf;     /* ACR Decrease Time Factor (10-bit) */
	int    cdf;      /* Cutoff Decrease Factor (3-bit) */
	int    spare;      /* spare bits */
	double tat;        /* future arrival time of a cell on  a connection */

protected:
	int command(int argc, const char*const* argv);
	static AtmTrafprm* instance_;
	int set_cbr(const char*const*,Tcl &);
	int set_rt_vbr(const char*const*, Tcl &);
	int set_nrt_vbr(const char*const*, Tcl &);
	int set_ubr(const char*const*, Tcl &);
	int set_abr(const char*const*, Tcl &);
	int set_gfr(const char*const*, Tcl &);
	void initialize();


};

class AtmQos {
public:
	AtmQos(){ }
	AtmQos(AtmTrafprm *rx,  AtmTrafprm *tx);
	AtmTrafprm *get_txtp() { return txtp; }
	AtmTrafprm *get_rxtp() { return rxtp; }

private:
	AtmTrafprm *rxtp;
	AtmTrafprm *txtp;
};


#endif





