/* -*- Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 2000 Regents of the University of New Hampshire.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the MASH Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /opt/src/mash/repository/vint/ns-2/atmscheduler.h,v 1.0 2000/07/22 22:46 sow Exp $
 *
 * Contributed by Sowmya Manjanatha, Dept. of Computer Science, UNH, July 2000 
 */

#ifndef __atmscheduler_h__
#define __atmscheduler_h__

#include "connector.h"
#include "packet.h"
#include "atmqueue.h"
#include "atm.h"
#include "atmpvc.h"

#define NUM_CATEGORIES 6
#define MAX_Q 20
#define CELLS_PER_ROUND 1


char qosT[NUM_CATEGORIES][10] = {"cbr", "rt-vbr", "nrt-vbr", "abr", "ubr",
                                 "gfr"};

class AtmScheduler;

class AtmSchedulerHandler : public Handler {
public:
	AtmSchedulerHandler(AtmScheduler *scheduler_i) : scheduler(scheduler_i) { }
	virtual ~AtmSchedulerHandler() {}
	void handle(Event *e);

private:
	AtmScheduler *scheduler;
	
};


class AtmScheduler : public Connector {
public:
	AtmScheduler();
	void recv(Packet *p, Handler *);
	void resume();
	void reset();

protected:
	AtmSchedulerHandler handler;
	int command(int argc, const char*const* argv);
	bool enque(Packet *p);
	int pvc_found(int,int,int);
	void assign_service_sizes_per_category();
	void update_total_input_rate(int,int);
	Packet *deque();
	Packet *deque_old();
	Packet *priority_scheduler();
	Packet *pri_vc_scheduler(int);
	Packet *per_vc_queuing(int,int,int,int &);
	Packet *wrr_scheduler();
	Packet *wrr_vc_scheduler(int);
	Packet *WRR_vc_scheduler(int);
        bool gcra_helper(Packet *,int);
        bool gcra(double T, double tau, int c, int q); 
        int set_qlength(const char*const*);
        int set_qweight(const char*const*);
        void cells_per_q(int);
	
	bool idle_;
	
private:

	vector<AtmQueue> per_vc_q[NUM_CATEGORIES];
	int cbr_qlength;
	int rt_vbr_qlength;
	int nrt_vbr_qlength;
	int gfr_qlength;
	int abr_qlength;
	int ubr_qlength;
        int qlen_[NUM_CATEGORIES][MAX_Q];
        double tat_[NUM_CATEGORIES][MAX_Q];
        int wrrTemp[NUM_CATEGORIES][MAX_Q];
        int qWeight[NUM_CATEGORIES][MAX_Q];

	double cbr_qweight;
	double rt_vbr_qweight;
	double nrt_vbr_qweight;
	double gfr_qweight;
	double abr_qweight;
	double ubr_qweight;

	int bytes_per_round;
	pvc_db *pvc_dbase;
	int cells_per_category[6];
	double total_input_rate;
	double fair_share;
	double vcshare;
	double er;
	double lf;
	int last_served_category;
	int last_served_queue_in_category[NUM_CATEGORIES];
        int last_served_q;
        int cells_per_round[MAX_Q];



	//statistics
	
	unsigned int rx_cells;
	unsigned int tx_cells;
	unsigned int drops;
	unsigned int undequeued;

};

#endif




