#
# Copyright (c) 2000, CNRG, University of New Hampshire.
# All rights reserved.
#
# License is granted to copy, to use, and to make and to use derivative
# works for research and evaluation purposes.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Contributed by Sowmya Manjanatha <sowmyam@cs.unh.edu>
#
#

# insert the conditioner between the end-station and the
# ATMNode.

# a simple atm link (written to substitue the default with the AtmScheduler)
Simulator instproc atm-simplex-link {n1 n2 bw delay pvc_dbase args} {
    global packetsize linkrate rate dly bw_

    
    set dly $delay 
    set bw_ $bw

    $self instvar link_
    set sid [$n1 id]
    set did [$n2 id]
    set scheduler [new AtmScheduler]

    set rate [$self bw_parse $bw]
    set dly [$self delay_parse $delay]
    $scheduler set total_service_rate_ $rate
    puts "scheduler = $scheduler"
    set link_($sid:$did) [new SimpleLink $n1 $n2 $bw $delay $scheduler]
    $n1 add-neighbor $n2

    set trace [$self get-ns-traceall]
    if {$trace != ""} {
	$self trace-queue $n1 $n2 $trace
    }
    set trace [$self get-nam-traceall]
    if {$trace != ""} {
	$self namtrace-queue $n1 $n2 $trace
    }
    $self register-nam-linkconfig $link_($sid:$did)
    $scheduler attach-pvc_dbase $pvc_dbase
    $pvc_dbase instvar linknum_
    $pvc_dbase add-resources $rate $dly $linknum_

    # the scheduler could access the pvc_dbase to be assigned to the
    # atm node later.  For now, I will  attach the scheduler to the
    # atm node.

    return $scheduler

}

# an atm duplex-link

Simulator instproc atm-duplex-link {n1 n2 bw delay pvc_dbase pvc_dbase1 args} {

    $n1 instvar scheduler
    
    set scheduler(0) [ $self atm-simplex-link $n1 $n2 $bw $delay $pvc_dbase $args]
    set scheduler(1) [ $self atm-simplex-link $n2 $n1 $bw $delay $pvc_dbase1 $args]
    
}


# nam support for link orientation, color etc.

Simulator instproc atm-duplex-link-op { n1 n2 op args } {
    $self instvar link_
    eval $link_([$n1 id]:[$n2 id]) $op $args
    eval $link_([$n2 id]:[$n1 id]) $op $args
}

Node instproc install_pvc_database {} {
    $self instvar classifier_
    set pvc_database [new pvc_db]
    $classifier_ cmd configure_atmswitch $pvc_database

    return $pvc_database
}


AtmScheduler instproc attach-traces {src dst file {op ""}} {
	#Do nothing here
}


AtmScheduler instproc attach-nam-traces {src dst file} {
    
	#valid only if the default trace type in attach-traces is Drop, Enque, Deque.
	#see comment above.
	#this function should be different for different queue when needed.
	
	$self attach-traces $src $dst $file "nam"
}

# helper function for atmconditioner
SimpleLink instproc get-target {} {
    $self instvar link_
    set tg [$link_ target]
    return $tg
}

# helper function for atmconditioner
SimpleLink instproc set-target {tg} {
    $self instvar link_
    $link_ target $tg
}


# insert the conditioner between the end-station and the
# ATMNode.

Simulator instproc atmconditioner {cond n1 atmn pvc_dbase bucketrate qlen} {


    $self instvar link_
    set sid [$n1 id]
    set did [$atmn id]
    set temp $link_($sid:$did)

    # The conditioner encompasses two modules, the CAC, UPC module and
    # the traffic shaper (token bucket) module.  Therefore the
    # linktarget is preceeded by two other modules, the AtmConditioner
    # and the TBF (token bucket shaper) module.

    $n1 instvar tbf_
    for {set i 0} {$i < 1} {incr i} {
       set tbf($i) [new TBF]
       $tbf($i) set qlen_ $qlen
       $n1 set tbf_($i) $tbf($i)
    }

    set tbfrate [$self bw_parse $bucketrate]

    $tbf(0) set rate_ $tbfrate 
#    $tbf(1) set rate_ $tbfrate


    set linktarget [$temp get-target]
    $temp set-target $cond
#    $cond target $linktarget
    $cond target $tbf(0)
    $tbf(0) target $linktarget

    $atmn instvar classifier_
    $classifier_ cmd is_atmswitch

# this is to make future reference to the conditioner available 
    $cond add-pvc_dbase $pvc_dbase
    set bw [$self bw_parse $bucketrate]

#    $cond total_service_rate_ $bw

    $pvc_dbase attach-conditioner $cond

}


#attaching the aal_driver in between agent and classifier

Simulator instproc merge_source_sardriver {source_agent source dest pvc_dbase bw_ dly} {
    
    $source instvar sar_
    $sar_ configure_atmswitch $pvc_dbase
    
    set agenttarget [$source_agent target]
    
    $source_agent target $sar_
    $sar_ target $agenttarget 
    
    # set the bandwidth and the delay required for scheduling the packets on SAR
    
    $sar_ start_timer [$self delay_parse $dly] [$self bw_parse $bw_]
    
    return $pvc_dbase

} 

# adding an ATM virtual connection between an end-station and 
# the ATM Switch

Node instproc add-pvc { vpi vci rxtp txtp type } {
    
    $self instvar sar_
    $self instvar classifier_
    $classifier_ cmd attach_aal5 $vpi $vci $rxtp $txtp $type
   
    set sar_ [new Agent/SARDriver]
    $sar_ attach_aal5 $vpi $vci $rxtp $txtp $type
    
}



# attaching the destination agent through SAR

Simulator instproc attach-agent-thro-sar { dest_node dest_agent source_agent} {

    $dest_node instvar sar_

    $self attach-agent $dest_node $sar_

    $self connect $source_agent $sar_
    $self connect $sar_ $dest_agent

}


Simulator instproc add_edge_sardriver {edge sar_e pvc_dbase bw_ dly} {
    
    $sar_e instvar sar_
    $sar_ configure_atmswitch $pvc_dbase

    $self instvar link_
    set sid [$edge id]
    set did [$sar_e id]
    set temp $link_($sid:$did)
    set linktarget [$temp get-target]

    $temp set-target $sar_
    $sar_ target $linktarget

    # set the bandwidth and the delay required for scheduling the packets on SAR
    
    $sar_ start_timer [$self delay_parse $dly] [$self bw_parse $bw_]
    
    return $pvc_dbase

} 

